@@ -61,7 +61,7 @@
 #endif
 #endif
 
-#ifdef __linux
+#ifdef __linux__
 /*for monitoring total space allocated via malloc*/
 #include <malloc.h>
 #endif
@@ -103,9 +103,10 @@ static char * tag_name = NULL;
 static char * expected_res = NULL;
 static size_t max_vm_kb = 0;
 static int run_skipped_tests = 0;
-//0 if deactivated, or > 0, representing the maximum number of subprocesses to launch
-static int parallel_suites = 0;
+static int max_parallel_suites = 0; /*if 0, but parallel is requested, an arbitrary value is determined.*/
+static int run_in_parallel = 0;
 static uint64_t globalTimeout = 0;
+static int max_failed_tests_threshold = 0;
 
 //To keep record of the process name who started and args
 static char **origin_argv = NULL;
@@ -116,6 +117,9 @@ static int (*verbose_arg_func)(const char *arg) = NULL;
 static int (*silent_arg_func)(const char *arg) = NULL;
 static int (*logfile_arg_func)(const char *arg) = NULL;
 
+//Processing custom native events for processing events (like PeekMessage for Windows) that is not implemented by Linphone 
+static void (*process_events)(void) = NULL;
+
 void bc_tester_set_silent_func(int (*func)(const char*)) {
 	if (func) {
 		silent_arg_func = func;
@@ -134,6 +138,15 @@ void bc_tester_set_logfile_func(int (*func)(const char*)) {
 	}
 }
 
+void bc_tester_set_process_events_func(void (*func)(void)) {
+	process_events = func;
+}
+
+void bc_tester_process_events(){
+	if( process_events)
+		process_events();
+}
+
 static void (*tester_printf_va)(int level, const char *format, va_list args)=NULL;
 
 void bc_tester_printf(int level, const char *format, ...) {
@@ -314,20 +327,6 @@ char *bc_tester_get_failed_asserts(void) {
 	return buffer;
 }
 
-#ifdef _WIN32
-
-void write_suite_result_file(char *suite_name, char *results_string) {
-	(void)suite_name;
-	(void)results_string;
-	//TODO Windows support
-}
-
-void merge_and_print_results_files(void) {
-	//TODO Windows support
-}
-
-#else
-
 void write_suite_result_file(char *suite_name, char *results_string) {
 	bctbx_vfs_file_t* bctbx_file;
 	char *suite_name_wo_spaces, *file_name;
@@ -359,7 +358,7 @@ void merge_and_print_results_files(void) {
 		bctbx_file = bctbx_file_open2(bctbx_vfs_get_default(), file_name, O_RDONLY);
 
 		if (bctbx_file) {
-			file_size = (int64_t) bctbx_file_size(bctbx_file);
+			file_size = (ssize_t) bctbx_file_size(bctbx_file);
 			if (file_size > 0) {
 				buffer = malloc(file_size + 1);
 				read_bytes = bctbx_file_read(bctbx_file, (void *)buffer, file_size, 0);
@@ -401,11 +400,9 @@ void merge_and_print_results_files(void) {
 	}
 }
 
-#endif
-
 static void all_complete_message_handler(const CU_pFailureRecord pFailure) {
 #ifdef HAVE_CU_GET_SUITE
-	if (parallel_suites != 0) {
+	if (run_in_parallel != 0) {
 		if (suite_name) {
 			char *results = CU_get_run_results_string();
 			write_suite_result_file(suite_name, results);
@@ -476,7 +473,7 @@ static void test_complete_message_handler(const CU_pTest pTest, const CU_pSuite
 	//insert empty line
 	bc_tester_printf(bc_printf_verbosity_info,"");
 
-#ifdef __linux
+#ifdef __linux__
 	/* use mallinfo() to monitor allocated space. It is linux specific but other methods don't work:
 	 * setrlimit() RLIMIT_DATA doesn't count memory allocated via mmap() (which is used internally by malloc)
 	 * setrlimit() RLIMIT_AS works but also counts virtual memory allocated by thread stacks, which is very big and
@@ -556,31 +553,36 @@ void merge_junit_xml_files(const char *dst_file_name) {
 			if (read_bytes == file_size) {
 				total_size += file_size;
 				suite_junit_xml_results[i][file_size] = '\0';
+				//Also remove the file
+				bctbx_file_close(bctbx_file);
+				remove(file_name);
 			} else {
 				bc_tester_printf(bc_printf_verbosity_error, "Could not read JUnit XML file '%s' to merge", file_name);
 				bctbx_free(suite_junit_xml_results[i]);
 				suite_junit_xml_results[i] = NULL;
+				bctbx_file_close(bctbx_file);
 			}
 		} else {
 			bc_tester_printf(bc_printf_verbosity_error, "Could not open JUnit XML file '%s' to merge", file_name);
+			suite_junit_xml_results[i] = NULL;
+			bctbx_file_close(bctbx_file);
 		}
-		bctbx_file_close(bctbx_file);
-		//Also remove the file
-		remove(file_name);
 		bctbx_free(file_name);
 	}
 	//Empty the destination file
 	bctbx_file = bctbx_file_open(bctbx_vfs_get_default(), dst_file_name, "w+");
-	bctbx_file_truncate(bctbx_file, 0);
-	offset = bctbx_file_fprintf(bctbx_file, 0, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<testsuites>\n");
-	for (i = 0; i < nb_test_suites; i++) {
-		if (suite_junit_xml_results[i] != NULL) {
-			offset += bctbx_file_fprintf(bctbx_file, offset, suite_junit_xml_results[i]);
-			bctbx_free(suite_junit_xml_results[i]);
+	if(bctbx_file){
+		bctbx_file_truncate(bctbx_file, 0);
+		offset = bctbx_file_fprintf(bctbx_file, 0, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<testsuites>\n");
+		for (i = 0; i < nb_test_suites; i++) {
+			if (suite_junit_xml_results[i] != NULL) {
+				offset += bctbx_file_fprintf(bctbx_file, offset, suite_junit_xml_results[i]);
+				bctbx_free(suite_junit_xml_results[i]);
+			}
 		}
+		bctbx_file_fprintf(bctbx_file, offset, "</testsuites>\n");
+		bctbx_file_close(bctbx_file);
 	}
-	bctbx_file_fprintf(bctbx_file, offset, "</testsuites>\n");
-	bctbx_file_close(bctbx_file);
 	bctbx_free(suite_junit_xml_results);
 }
 
@@ -601,7 +603,7 @@ void merge_log_files(const char *base_logfile_name) {
 	for (i = 0; i < nb_test_suites; ++i) {
 		char *suite_logfile_name = get_logfile_name(log_file_name, test_suite[i]->name);
 		bctbx_file = bctbx_file_open2(bctbx_vfs_get_default(), suite_logfile_name, O_RDONLY);
-
+		
 		if (!bctbx_file) {
 			bc_tester_printf(bc_printf_verbosity_error, "Could not open log file '%s' to merge into '%s'", suite_logfile_name, base_logfile_name);
 			continue;
@@ -622,9 +624,22 @@ void merge_log_files(const char *base_logfile_name) {
 }
 
 //Number of test suites to run concurrently
-//TODO better default or add cli option ?
 int bc_tester_get_max_parallel_processes(void) {
-	return (nb_test_suites / 2) + 1;
+	if (max_parallel_suites == 0)
+		return (nb_test_suites / 2) + 1;
+	return MIN(max_parallel_suites, nb_test_suites);
+}
+
+void bc_tester_set_max_parallel_suites(int nb_suites){
+	max_parallel_suites = nb_suites;
+}
+
+void bc_tester_set_max_failed_tests_threshold(int threshold){
+	max_failed_tests_threshold = threshold;
+}
+
+void bc_tester_set_global_timeout(int seconds){
+	globalTimeout = seconds;
 }
 
 #ifdef _WIN32
@@ -648,8 +663,72 @@ void kill_sub_processes(int *pids) {
 #endif
 
 #ifdef _WIN32
-int start_sub_process(const char *suite_name) {
-	//TODO Windows support
+
+//Start	test subprocess for the given suite
+int start_sub_process(const char *suite_name, PROCESS_INFORMATION * pi) {
+	int argc = 0;
+	int i;
+	//const char *argv[origin_argc + 10]; //Assume safey 10 more parameters
+	char * commandLine = bctbx_strdup(origin_argv[0]);
+	char * commandLineTemp = NULL;
+	bool_t propagateXmlFileSetter = TRUE;
+	bool_t propagateLogFileSetter = log_file_name!=NULL;// Log file has not been set : Don't propagate it
+
+	for (i = 1; origin_argv[i]; ++i) {
+		if (strcmp(origin_argv[i], "--xml-file") == 0) {
+			propagateXmlFileSetter = FALSE;
+		}
+		if (strcmp(origin_argv[i], "--log-file") == 0) {
+			//Create a specific log file for this suite
+			commandLineTemp = commandLine;
+			commandLine = bc_sprintf("%s %s \"%s\"", commandLineTemp, origin_argv[i], get_logfile_name(log_file_name, suite_name) );
+			bctbx_free(commandLineTemp);
+			propagateLogFileSetter = FALSE;
+			++i;
+		} else {
+			//Keep other parameters
+			commandLineTemp = commandLine;
+			commandLine = bc_sprintf("%s %s", commandLineTemp, origin_argv[i] );
+			bctbx_free(commandLineTemp);
+		}
+	}
+	commandLineTemp = commandLine;
+	commandLine = bc_sprintf("%s --xml --child --suite \"%s\"", commandLineTemp, suite_name);
+	bctbx_free(commandLineTemp);
+	if( propagateXmlFileSetter){// Add current xml_file to child commands
+		commandLineTemp = commandLine;
+		commandLine = bc_sprintf("%s --xml-file \"%s\"", commandLineTemp, xml_file);
+		bctbx_free(commandLineTemp);
+	}
+	if(propagateLogFileSetter){
+		commandLineTemp = commandLine;
+		commandLine = bc_sprintf("%s --log-file \"%s\"", commandLineTemp, get_logfile_name(log_file_name, suite_name) );
+		bctbx_free(commandLineTemp);
+	}
+	
+	// Start the child process.
+	STARTUPINFOA si;
+
+	ZeroMemory( &si, sizeof(si) );
+	si.cb = sizeof(si);
+	ZeroMemory( pi, sizeof(*pi) );
+	if( !CreateProcessA( NULL,   // No module name (use command line)
+		commandLine,        // Command line
+		NULL,           // Process handle not inheritable
+		NULL,           // Thread handle not inheritable
+		FALSE,          // Set handle inheritance to FALSE
+		0,//CREATE_NEW_CONSOLE,              // No creation flags
+		NULL,           // Use parent's environment block
+		NULL,           // Use parent's starting directory
+		&si,            // Pointer to STARTUPINFO structure
+		pi )           // Pointer to PROCESS_INFORMATION structure
+    )
+    {
+        printf( "CreateProcess failed (%d).\n", GetLastError() );
+        return GetLastError();
+    }
+	bctbx_free(commandLine);
+
 	return 0;
 }
 
@@ -693,49 +772,125 @@ int start_sub_process(const char *suite_name) {
 //For parallel tests only - handle anormally exited test suites
 //Remove previously generated XML suite file if exited anormally (could cause unusable final JUnit XML)
 //And mark all tests for the suite as failed
-int handle_sub_process_error(int pid, int exitStatus, int *suitesPids) {
-	if (abs(exitStatus) > 1) {
-		int i, j;
-		for (i = 0; i < nb_test_suites; ++i) {
-			if (suitesPids[i] == pid) {
-				ssize_t offset;
-				char *suite_file_name = get_junit_xml_file_name(test_suite[i]->name, "-Results.xml");
-				bctbx_vfs_file_t* bctbx_file = bctbx_file_open(bctbx_vfs_get_default(), suite_file_name, "w+");
-				bctbx_file_truncate(bctbx_file, 0);
+int handle_sub_process_error(int pid, int *suitesPids) {
+	int failed_tests = 0;
+	int i, j;
+	for (i = 0; i < nb_test_suites; ++i) {
+		if (suitesPids[i] == pid) {
+			ssize_t offset;
+			char *suite_file_name = get_junit_xml_file_name(test_suite[i]->name, "-Results.xml");
+			bctbx_vfs_file_t* bctbx_file = bctbx_file_open(bctbx_vfs_get_default(), suite_file_name, "w+");
+			bctbx_file_truncate(bctbx_file, 0);
 
-				offset = bctbx_file_fprintf(bctbx_file, 0, "\n<testsuite name=\"%s\" tests=\"%d\" time=\"0\" failures=\"%d\" errors=\"0\" skipped=\"0\">\n", test_suite[i]->name, test_suite[i]->nb_tests, test_suite[i]->nb_tests);
-				for (j=0; j < test_suite[i]->nb_tests; ++j) {
-					offset += bctbx_file_fprintf(bctbx_file, offset, "\t<testcase classname=\"%s\" name=\"%s\">\n", test_suite[i]->name, test_suite[i]->tests[j].name);
-					offset += bctbx_file_fprintf(bctbx_file, offset, "\t\t<failure message=\"\" type=\"Failure\">\n\t\tGlobal suite failure\n");
-					offset += bctbx_file_fprintf(bctbx_file, offset, "\t\t</failure>\n\t</testcase>\n");
-				}
-				bctbx_file_fprintf(bctbx_file, offset, "\n</testsuite>\n");
-				bc_tester_printf(bc_printf_verbosity_info, "Suite '%s' ended in error. Marking all tests as failed", test_suite[i]->name);
-				bctbx_file_close(bctbx_file);
-				bctbx_free(suite_file_name);
+			offset = bctbx_file_fprintf(bctbx_file, 0, "\n<testsuite name=\"%s\" tests=\"%d\" time=\"0\" failures=\"%d\" errors=\"0\" skipped=\"0\">\n", test_suite[i]->name, test_suite[i]->nb_tests, test_suite[i]->nb_tests);
+			failed_tests = test_suite[i]->nb_tests;
+			for (j=0; j < test_suite[i]->nb_tests; ++j) {
+				offset += bctbx_file_fprintf(bctbx_file, offset, "\t<testcase classname=\"%s\" name=\"%s\">\n", test_suite[i]->name, test_suite[i]->tests[j].name);
+				offset += bctbx_file_fprintf(bctbx_file, offset, "\t\t<failure message=\"\" type=\"Failure\">\n\t\tGlobal suite failure\n");
+				offset += bctbx_file_fprintf(bctbx_file, offset, "\t\t</failure>\n\t</testcase>\n");
 			}
+			bctbx_file_fprintf(bctbx_file, offset, "\n</testsuite>\n");
+			bc_tester_printf(bc_printf_verbosity_info, "Suite '%s' ended in error. Marking all tests as failed", test_suite[i]->name);
+			bctbx_file_close(bctbx_file);
+			bctbx_free(suite_file_name);
+			break;
 		}
 	}
-	return exitStatus;
+	return failed_tests;
 }
 
 #ifdef _WIN32
-//TODO Windows support
 int bc_tester_run_parallel(void) {
-	return 0;
+	int ret = 0; //Global return status;
+	if( nb_test_suites > 0){
+		PROCESS_INFORMATION * suitesPids = malloc(nb_test_suites * sizeof( PROCESS_INFORMATION));
+		int *processed = malloc(nb_test_suites * sizeof(int));
+		uint64_t time_start = bctbx_get_cur_time_ms(), elapsed = time_start, print_timer = time_start;
+
+		//Assume there is a problem if a suite is still running 60mn after the start of the tester. TODO make timeout	a cli parameter ?
+		uint64_t timeout = 0;
+		if (globalTimeout <= 0) {
+				globalTimeout = 60 * 60;
+		}
+		timeout = time_start + (globalTimeout * 1000);
+
+
+		int maxProcess = bc_tester_get_max_parallel_processes();
+		int nextSuite = 0; //Next suite id to be exec'd
+		int runningSuites = 0; //Number of currently running suites
+		int testsFinished = 0;
+
+		memset(suitesPids, 0, nb_test_suites*sizeof(PROCESS_INFORMATION));
+		memset(processed, 0, nb_test_suites*sizeof(int));
+		do {
+			if (nextSuite < nb_test_suites && runningSuites < maxProcess) {
+				PROCESS_INFORMATION pi;
+
+				if (start_sub_process(test_suite[nextSuite]->name, &pi) != 0) {
+					bc_tester_printf(bc_printf_verbosity_error, "Error while starting suite sub-process. Aborting.");
+					return -1;
+				}
+				suitesPids[nextSuite] = pi;
+				runningSuites++;
+				nextSuite++;
+			}else{
+				for(int i = 0 ;  i < nextSuite ; ++i){
+					if( processed[i] == 0){
+						int returnCode = WaitForSingleObject( suitesPids[i].hProcess, 1000 );
+						if( ( returnCode != WAIT_TIMEOUT)){
+							--runningSuites;
+							++testsFinished;
+							bc_tester_printf(bc_printf_verbosity_error, "Suite sub process (ID %d) terminated with return code %d.", i, returnCode);
+							processed[i] = 1;
+							ret += returnCode;
+						}
+					}
+				}
+			}
+			bctbx_sleep_ms(50);
+			if (elapsed - print_timer > 10000) { //print message only every ~10s...
+				bc_tester_printf(bc_printf_verbosity_error, "Waiting for test suites to finish... Total Suites(%d). Suites running(%d), Finished(%d)", nb_test_suites, runningSuites, testsFinished);
+				print_timer = bctbx_get_cur_time_ms();
+			}
+			elapsed = bctbx_get_cur_time_ms();
+		} while (testsFinished < nb_test_suites && elapsed < timeout);
+
+		if (elapsed >= timeout) {
+			bc_tester_printf(bc_printf_verbosity_error, "Stopped waiting for all test suites to execute as we reach timeout. Killing running suites.");
+			for(int i = 0 ;  i < nextSuite ; ++i){
+				if( processed[i] == 0)
+					TerminateProcess(suitesPids[i].hProcess, -1);
+			}
+			bc_tester_printf(bc_printf_verbosity_error, "*** Test suite took too much time. Please check errors or split longest test suites to benefit from parallel execution. ***");
+			ret = -1;
+		}
+		for(int i = 0 ;  i < nextSuite ; ++i){
+			CloseHandle( suitesPids[i].hProcess );
+			CloseHandle( suitesPids[i].hThread );
+		}
+		bc_tester_printf(bc_printf_verbosity_info, "All suites ended.");
+		
+		if (ret != -1){
+			int seconds = (int)(elapsed - time_start)/1000;
+			all_complete_message_handler(NULL);
+			bc_tester_printf(bc_printf_verbosity_info, "Full parallel run completed in %2i mn %2i s.\n", seconds/60, seconds % 60);
+		}
+		free( suitesPids);
+		free( processed);
+	}
+	return ret == -1 ?  ret : (ret > max_failed_tests_threshold);
 }
 #else
 
 int bc_tester_run_parallel(void) {
 	int suitesPids[nb_test_suites];
 	uint64_t time_start = bctbx_get_cur_time_ms(), elapsed = time_start, print_timer = time_start;
-
-	//Assume there is a problem if a suite is still running 60mn after the start of the tester. TODO make timeout	a cli parameter ?
 	uint64_t timeout = 0;
+	
 	if (globalTimeout <= 0) {
-			globalTimeout = 60;
+			globalTimeout = 60 * 60;
 	}
-	timeout = time_start + (globalTimeout * 60 * 1000);
+	timeout = time_start + (globalTimeout * 1000);
 
 
 	int maxProcess = bc_tester_get_max_parallel_processes();
@@ -775,14 +930,11 @@ int bc_tester_run_parallel(void) {
 					++testsFinished;
 				}
 				if (WIFSIGNALED(wstatus)) {
-					childRet = WTERMSIG(wstatus);
+					childRet = handle_sub_process_error(childPid, suitesPids);
 				} else {
 					childRet = WEXITSTATUS(wstatus);
 				}
-				handle_sub_process_error(childPid, childRet, suitesPids);
-				if (ret == 0 &&	childRet != 0) {
-					ret = childRet;
-				}
+				ret += childRet;
 				bc_tester_printf(bc_printf_verbosity_error, "Suite sub process (pid %d) terminated with return code %d.", childPid, childRet);
 			}
 		}
@@ -796,16 +948,19 @@ int bc_tester_run_parallel(void) {
 
 	if (elapsed >= timeout) {
 		bc_tester_printf(bc_printf_verbosity_error, "Stopped waiting for all test suites to execute as we reach timeout. Killing running suites.");
+		bc_tester_printf(bc_printf_verbosity_error, "*** Test suite took too much time. Please check errors or split longest test suites to benefit from parallel execution. ***");
 		kill_sub_processes(suitesPids);
+		ret = -1;
 	}
 	bc_tester_printf(bc_printf_verbosity_info, "All suites ended.");
-	all_complete_message_handler(NULL);
-	{
+	
+	if (ret != -1){
 		int seconds = (int)(elapsed - time_start)/1000;
-		
+		all_complete_message_handler(NULL);
 		bc_tester_printf(bc_printf_verbosity_info, "Full parallel run completed in %2i mn %2i s.\n", seconds/60, seconds % 60);
+		return ret > max_failed_tests_threshold;
 	}
-	return ret;
+	return -1;
 }
 
 #endif
@@ -827,11 +982,11 @@ int bc_tester_run_tests(const char *suite_name, const char *test_name, const cha
 	CU_set_suite_init_failure_handler(suite_init_failure_message_handler);
 	CU_set_suite_cleanup_failure_handler(suite_cleanup_failure_message_handler);
 
-	if (xml_enabled == 1) {
-		char *xml_file_name;
+	if (xml_enabled == 1 && !test_name) {
+		char *xml_file_name;//, *xml_file_name_tmp;
 		CU_automated_enable_junit_xml(TRUE); /* this requires 3.0.1 because previous versions crash automated.c */
 
-		if (parallel_suites != 0) {
+		if (run_in_parallel != 0) {
 			//Sub-process started by parent in bc_tester_run_parallel
 			if (suite_name) {
 				CU_automated_enable_partial_junit(TRUE);
@@ -841,11 +996,14 @@ int bc_tester_run_tests(const char *suite_name, const char *test_name, const cha
 				CU_automated_run_tests();
 			} else { //Starting registered suites in parallel
 				ret = bc_tester_run_parallel();
-				xml_file_name = get_junit_xml_file_name(NULL, "-Results.xml");
-				merge_junit_xml_files(xml_file_name);
-				bctbx_free(xml_file_name);
-				if (log_file_name) {
-					merge_log_files(log_file_name);
+				if (ret != -1){
+					/* -1 means timeout, in this case don't generate junit report. */
+					xml_file_name = get_junit_xml_file_name(NULL, "-Results.xml");
+					merge_junit_xml_files(xml_file_name);
+					bctbx_free(xml_file_name);
+					if (log_file_name) {
+						merge_log_files(log_file_name);
+					}
 				}
 				return ret;
 			}
@@ -908,12 +1066,21 @@ int bc_tester_run_tests(const char *suite_name, const char *test_name, const cha
 					CU_run_all_tests();
 				}
 		}
-#ifdef __linux
+#ifdef __linux__
 	bc_tester_printf(bc_printf_verbosity_info, "Still %i kilobytes allocated when all tests are finished.",
 			 mallinfo().uordblks / 1024);
 #endif
-
-	return CU_get_number_of_tests_failed()!=0;
+	if (run_in_parallel){
+		// We are a child process, return the number of test failed.
+		int failed_tests = CU_get_number_of_tests_failed();
+		if (failed_tests >= 255) {
+			bc_tester_printf(bc_printf_verbosity_error, "The number of tests exceeded 255, the maximum value for an exit status !");
+			failed_tests = 255;
+		}
+		return failed_tests;
+	}
+	/* Otherwise it is serialized execution.*/
+	return (int)CU_get_number_of_tests_failed() > (int)max_failed_tests_threshold;
 
 }
 
@@ -1080,7 +1247,7 @@ void bc_tester_init(void (*ftester_printf)(int level, const char *format, va_lis
 }
 
 void bc_tester_set_max_vm(size_t amax_vm_kb) {
-#ifdef __linux
+#ifdef __linux__
 	max_vm_kb = (size_t)amax_vm_kb;
 	bc_tester_printf(bc_printf_verbosity_info, "Maximum virtual memory space set to %li kilo bytes", max_vm_kb);
 #else
@@ -1110,6 +1277,7 @@ void bc_tester_helper(const char *name, const char* additionnal_helper) {
 			 "\t\t\t--max-alloc <size in ko> (maximum amount of memory obtained via malloc allocator)\n"
 			 "\t\t\t--max-alloc <size in ko> (maximum amount of memory obtained via malloc allocator)\n"
 			 "\t\t\t--parallel (Execute tests concurrently and with JUnit report)\n"
+			 "\t\t\t--parallel-max (Number Max of parallel processes)\n"
 			 "\t\t\t--timeout <timeout in minutes> (sets the global timeout when used alongside to the parallel option, the default value is 60)\n"
 			 "And additionally:\n"
 			 "%s",
@@ -1168,10 +1336,13 @@ int bc_tester_parse_args(int argc, char **argv, int argid)
 		}
 		//Defaults to JUnit report if parallel is enabled
 		xml_enabled = 1;
-		parallel_suites = 1;
+		run_in_parallel = 1;
+	} else if (strcmp(argv[i], "--parallel-max") == 0) {
+		CHECK_ARG("--parallel-max", ++i, argc);
+		bc_tester_set_max_parallel_suites(atoi(argv[i]));
 	} else if (strcmp(argv[i], "--timeout") == 0) {
 		CHECK_ARG("--timeout", ++i, argc);
-		globalTimeout = atoi(argv[i]);
+		globalTimeout = atoi(argv[i]) * 60;
 	} else if (strcmp(argv[i], "--max-alloc") == 0) {
 		CHECK_ARG("--max-alloc", ++i, argc);
 		max_vm_kb = atol(argv[i]);
@@ -1181,6 +1352,7 @@ int bc_tester_parse_args(int argc, char **argv, int argid)
 	} else if (strcmp(argv[i], "--writable-dir") == 0) {
 		CHECK_ARG("--writable-dir", ++i, argc);
 		bc_tester_writable_dir_prefix = strdup(argv[i]);
+	}else if(strcmp(argv[i],"--child")==0){//Switch off this parameter as it is used for external processing
 	} else {
 		bc_tester_printf(bc_printf_verbosity_error, "Unknown option \"%s\"", argv[i]);
 		return -1;
